        SUBROUTINE driverlin_VRRKM( GE, MX, MY, JMAX, DELE, JJ, &
                BL, UL, Step, Eto, E0, GW, CFR, SFR, MODEL, MLIN )


	IMPLICIT NONE

!! MX = Number of Grid Points
!! MY = Number of harmonic frequencies (3N-7), projecting out the Reaction Coordinate
!! BL = Lower Limit of bond length 
!! UL = Upper Limit of bond length 
!! Eto = Energy of Reactant (kcal/mol)
!! E0 = Energy of the lowest-lying intermediate (kcal/mol)
!! Step = step size used to locate a variational bottleneck.
!! JMAX = maxinum number of bins in master-equation
!! DELE = energy bin
!! GE = vector of sum of states for the bottleneck
!! GW = matrix with a dimension of (MX x (MY+4)) that contains harmonic frequencies, rot. constants, and barrier heights for Grid Points

	Double precision CON
	Parameter ( CON = 349.75479275231D0 )

        Integer MX, MY
        Integer JMAX, Ntp, JJ
        Double precision  GW(MX,MY+4), GE(JMAX)

	Double precision A(MX), X(MX), Y(MX), Y2(MX) 
	Integer I, J, K, NN
	Double precision XX, YY
	Double precision BL, UL, Step, Eto, Eb, E0
        Double precision DSS1, DELE, Eng, SFR, E1, E2
        Double precision BB(3)
	Double precision T(JMAX), AT(JMAX), T1(JMAX), T2(JMAX)
	Double precision SS(JMAX), DS(JMAX), SS1(JMAX)

	Double precision, Allocatable :: GSS(:,:)
	Double precision, Allocatable :: FREQ(:)
        Double precision, Allocatable :: GWF(:,:)

        CHARACTER*3 CFR
        CHARACTER*8 MODEL, MLIN

	DSS1 = (UL-BL)/(MX-1)

	NN = INT((UL-BL)/Step) + 1

	ALLOCATE ( GSS(NN,JMAX), FREQ(NN) )
	ALLOCATE ( GWF(NN,MY+4) )

	DO J=1, MY+4
		DO I=1, MX
			A(I) = GW(I,J)
			X(I) = BL + (I-1)*DSS1
		ENDDO
		CALL spline(X,A,MX,2.0d+30,2.0d+30,Y2)
		DO I=1, NN
			XX = BL + (I-1)*Step
			CALL splint(X,A,Y2,MX,XX,YY)
			GWF(I,J) = YY
		ENDDO
	ENDDO

	DO I=1, NN

        IF(CFR.EQ.'NO') THEN

		CALL TAT(T,AT,JMAX)
		DO J=1, MY
			FREQ(J) = GWF(I,J)
			CALL HO(T,AT,DELE,JMAX,FREQ(J))
		ENDDO

        ELSE IF(CFR.EQ.'YES') THEN

                CALL TAT(T1,AT,JMAX)
                DO J=1, MY-1
                        FREQ(J) = GWF(I,J)
                        CALL HO(T1,AT,DELE,JMAX,FREQ(J))
                ENDDO

                DO K=1, JMAX
                        SS1(K) = 0.0d0
                        DO J=1, K
                        E1 = (J-1)*DELE
                        SS1(K) = SS1(K) + T1(K-J+1)*sqrt(E1/GWF(I,MY))/SFR
                        ENDDO
                ENDDO
                T(1) = SS1(1)
                DO K=2, JMAX
                        T(K) = SS1(K) - SS1(K-1)
                ENDDO
        ELSE
                write(*,*) "INPUT FILE IS WRONG AT FREE-ROTOR"
                STOP
        ENDIF

		DO J=1, 3
			BB(J) = GWF(I,J+MY)
		ENDDO
		Eb = GWF(I,MY+4)

                Ntp = INT((Eto+Eb-E0)*CON/DELE) + 1  !!! 
	
		DO J=1, JMAX
			IF(J.LT.Ntp) THEN
				T2(J) = 0.0d0
			ELSE
				T2(J) = T(J-Ntp+1)
			ENDIF
		ENDDO

!!!                CALL cdsjlin(SS,DS,T2,AT,DELE,JMAX,BB(1),JJ) 
!!!                CALL cdsjlin(SS,DS,T2,AT,DELE,JMAX,BB,JJ) 

                CALL cdsj(SS,DS,T2,AT,DELE,JMAX,BB,JJ,MODEL,MLIN) 


		DO J=1, JMAX
			GSS(I,J) = SS(J)
			Eng = (J-1)*DELE
			DS(J) = 0.0d0
			SS(J) = 0.0d0
		ENDDO
	ENDDO

	DO J=1, JMAX
		GE(J) = GSS(1,J)
	ENDDO

	DO J=1, JMAX
		DO I=2, NN
			IF(GE(J).GT.GSS(I,J)) GE(J) = GSS(I,J)
		ENDDO
	ENDDO

	DEALLOCATE ( GSS, FREQ, GWF )

	RETURN
	END SUBROUTINE driverlin_VRRKM
	
