!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE CalA 
!CCC Purpose    : To build an A matrix in AX = B equation
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


	SUBROUTINE CalA(A,rKp,P,rktp,Maxi,KK,LL,NN,Np) 

!!!  A          : matrix A
!!!  rKp        : collision frequency (1/s) 
!!!  P          : energy transfer probability function
!!!  rktp       : microcanonical rate constants
!!!  Maxi       : maximum index used in master-equation
!!!  KK, LL     : index of intermediate
!!!  NN         : number of minima
!!!  Np         : number of products  


!!	IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)

        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)

        Integer I, J, K, L, MM, Maxi, KK, LL, NN, Np
real(kind=qp) :: rKp, A0, tp2(Maxi)
real(kind=qp) :: A(NN*(Maxi+Np),NN*(Maxi+Np)), P(NN,Maxi,Maxi), rktp(NN,NN+Np,Maxi)

        MM = NN*Maxi

	IF(KK.EQ.LL) THEN

        DO I=1, Maxi
                tp2(I) = 0.0d0
                DO J=1, Maxi
                        tp2(I) = tp2(I) + P(KK,J,I)
                ENDDO
        ENDDO

		DO I=1, Maxi
			DO J=1, Maxi
				IF(I.EQ.J) THEN
                        A0 = P(KK,I,I)-tp2(I)-rktp(KK,KK,I)/rKp
				ELSE
			A0 = P(KK,I,J)
				ENDIF
                        A(I+Maxi*(KK-1),J+Maxi*(KK-1)) = A0*rKp
			ENDDO
		ENDDO
	ELSE
		DO I=1, Maxi
			DO J=1, Maxi
				IF(I.EQ.J) THEN
					A(I+Maxi*(LL-1),J+Maxi*(KK-1))=rktp(KK,LL,I)
				ELSE
					A(I+Maxi*(LL-1),J+Maxi*(KK-1))=0.0D0
				ENDIF
			ENDDO
		ENDDO
	ENDIF

	RETURN
	END
