!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE CalCN 
!CCC Purpose    : To compute normalization factor in energy transfer
!CCC                    probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


        SUBROUTINE CalCN(CN,w,dE,AL,Ma,T,IND)
  
        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)
      
        Integer I, J, N, Ma, IND

        real(kind=qp) :: CN(Ma), w(Ma)
        real(kind=qp) :: dE, T, AL, beta, su, Puij, Pdij
        real(kind=qp) :: RT, su1, su2, tp, tpp1, tpp2, tpp3, tpp
        Parameter (RT=0.695038916D0)
        real(kind=qp) :: tpcn, Pd

        external Pdij

        beta=1.0D0/(RT*T)

        N=Ma

        su=0.0D0
        DO I=1, N
                tp = w(I)*w(N)
                IF(tp.NE.0.0D0) THEN
                su = su + exp(-(N-I)*dE/AL) 
                ENDIF
        ENDDO

        CN(N)=su

        DO J=N-1, 1, -1

                su1=0.0D0
                DO I=1, J
                        tp=w(I)*w(J)
                        IF(tp.NE.0.0D0) THEN
                        su1 = su1 + exp(-(J-I)*dE/AL)
                        ENDIF
                ENDDO

                su2 = 0.0d0
                IF(w(J).NE.0.0D0) THEN
                        DO I=J+1, N
                                Pd = Pdij(J,I,AL,CN,dE,w,N)
                                Pd = Pd*exp(-(I-J)*dE*beta) 
                                su2 = su2 + Pd*(w(I)/w(J))
                        ENDDO  
                ENDIF
                CN(J) = su1 / (1.0D0-su2)
                IF(CN(J).LT.0.0d0) CN(J) = 0.0d0
        ENDDO

!!        write(*,*) (".", I=1, 40)
!!        write(*,'(A35,I5)') "The INTERMEDIATE has been computed: ", IND
!!!        write(*,*) (".", I=1, 40)

!!        DO I=1, N
!!                write(999990,99) I, CN(I)
!!        ENDDO

99      FORMAT(2X,I5,2X,F15.5)

        RETURN
        END





