!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE CalP
!CCC Purpose    : To compute energy transfer probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE CalP(P,dE,AL,Maxi,T,Nsink,NN,w,IND)

!!!  P          : energy transfer probability function
!!!  dE         : energy bin (energy grain)
!!!  AL         : down energy (cm-1) 
!!!  Maxi       : maximum index in master-equation
!!!  T          : temperature
!!!  Nsink      : sink is set to collect stabilized populations of
!!!                     intermediates
!!!  NN         : number of intermediates
!!!  w          : density of states of considered intermediate
!!!  IND        : index of the lowest-lying intermediate

!!        IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
  
        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)

      
        Integer I, J, N, Maxi, n1, K, Nsink(NN), NN, IND

        real(kind=qp) :: CN(Maxi), E(Maxi), w(Maxi), C(Maxi)

        real(kind=qp) :: P(NN,Maxi,Maxi)

        real(kind=qp) :: EI, EJ, dE, T, AL, beta, su
        real(kind=qp) :: su1, su2, tp, tpp1, tpp2, tpp3, tpp
        real(kind=qp) :: Pdij, Puij, DIFF

        External Pdij, Puij

        call CalCN(CN,w,dE,AL,Maxi,T,IND)

        DO J=1, Maxi
                DO I=1, Maxi
                        IF(J.GE.I) THEN
                                P(IND,I,J) = Pdij(I,J,AL,CN,dE,w,Maxi)
                        ELSE
                                P(IND,I,J) = Puij(w,AL,CN,I,J,T,dE,Maxi)
                        ENDIF
                ENDDO
        ENDDO

        DO J=1, Maxi
                su = 0.0D0
                DO I=1, Maxi
                su = su + P(IND,I,J)
                ENDDO
!!!                write(22222,999) J, su

                DIFF = abs(su-1.0D0)

!!                IF((su.NE.0.0D0).and.(DIFF.GT.1.0D-06)) THEN
!!                        DO I=1, Maxi
!!                        write(20000+J,998) I, J, P(IND,I,J)
!!                        ENDDO
!!                ENDIF

        ENDDO
999     FORMAT (I5,2X,ES20.13)
998     FORMAT (I5,2X,I5,2X,ES20.13)

        RETURN
        END

