!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : FUNCTION Pdij 
!CCC Purpose    : To compute down energy transfer probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

!C=====================================================================================
!CCC     Computing a energy-transfer probable function
!CCC     using a simple exponential model
!C=====================================================================================

        REAL*8 FUNCTION Pdij(I,J,AL,CN,dE,w,Maxi)

        IMPLICIT NONE

!!!  J          : Index of bin; starting at J 
!!!  I          : Index of bin; ending at I
!!!  AL         : down energy transfer (cm-1)
!!!  CN         : normalization factor
!!!  dE         : energy bin (grain energy)
!!!  w          : density of states for considered intermediate
!!! Maxi        : maximum index in master-equation

        Integer I, J, Maxi

        REAL*8 CN(Maxi), w(Maxi)
        REAL*8 EI, EJ, AL, dE

!CCCCCCCCCC Down energy transfer from EJ to EI with EJ >= EI

       IF(w(J)*w(I).EQ.0.0D0) THEN
         Pdij = 0.0D0
!!       ELSE IF(CN(J).NE.0.0D0) THEN

       ELSE IF(CN(J).GT.0.0D0) THEN
         Pdij = exp(-(J-I)*dE/AL) / CN(J)
       ELSE
         Pdij = 0.0D0
       ENDIF

        RETURN
        END

