!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : FUNCTION Puij 
!CCC Purpose    : To compute up energy transfer probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


        REAL*8 FUNCTION Puij(w,AL,CN,I,J,T,dE,Maxi)
        IMPLICIT NONE

!!!  J          : Index of bin; starting at J 
!!!  I          : Index of bin; ending at I
!!!  AL         : down energy transfer (cm-1)
!!!  CN         : normalization factor
!!!  dE         : energy bin (grain energy)
!!!  w          : density of states for considered intermediate
!!!  Maxi       : maximum index in master-equation
!!!  T          : temperature
 
        Integer I, J, Maxi

        REAL*8 CN(Maxi), w(Maxi)
        REAL*8 EI, EJ, AL, Pdij, T, beta, dE
        REAL*8 Pd, RT
        Parameter (RT=0.695038916D0)

        External Pdij


!CCCCCCCCCC Up energy transfer from EJ to EI with EJ < EI


        beta=1.D0/(RT*T)

        IF(w(J)*w(I).EQ.0.0D0) THEN
            Puij=0.0D0
        ELSE

        EI=dE*(I-1)
        EJ=dE*(J-1)
                Pd=Pdij(J,I,AL,CN,dE,w,Maxi)
                Puij=Pd*exp(-beta*abs(EJ-EI))*w(I)/w(J)
        ENDIF

        RETURN
        END

