!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE cdsj 
!CCC Purpose    : To convolute vib. and rot. density and sum of states 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE cdsj(SS,DS,T,AT,DELE,Maxi,B,J,MODEL,MLIN)

        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)


!!!  SS         : sum of states at a pair (E,J)
!!!  DS         : density of states
!!!  T          : number of states within a bin
!!!  AT         : temperary vector
!!!  DELE       : energy bin (grain energy)
!!!  Maxi       : maximum index in master-equation
!!!  B          : three rot. constants
!!!  J          : angular momentum number

        integer Maxi, J
        real(kind=qp) :: SS(Maxi), DS(Maxi)
        integer K,I,N,JJ
        real(kind=qp) :: tp, B(3), Er, DELE, Ev(1000)
        real(kind=qp) :: T(Maxi), AT(Maxi)
        Character*8 MODEL, MLIN

        IF(MLIN.EQ."LINEAR") THEN

                CALL cdsjlin(SS,DS,T,AT,DELE,Maxi,B,J)

        ELSE 
                IF((MODEL.EQ."ACTIVE").OR.(MODEL.EQ."MODEL1")) THEN      !!! active rotors
                CALL cdsj_arot(SS,DS,T,AT,DELE,Maxi,B,J)
                ELSE IF((MODEL.EQ."INACTIVE").OR.(MODEL.EQ."MODEL2")) THEN  !!! inactive rotors
                CALL cdsj_inar(SS,DS,T,AT,DELE,Maxi,B,J)
                ELSE IF((MODEL.EQ."HYBRID").OR.(MODEL.EQ."MODEL3")) THEN   !!! hybrid rotors
                CALL cdsj_hybr(SS,DS,T,AT,DELE,Maxi,B,J)
                ELSE
        write(*,*) "Model chosen for External rotors is wrong !"
        write(*,*) "MODEL1=Active, MODEL2=Inactive, or MODEL3=Hybrid & 
                        only !"
                ENDIF
        ENDIF        

        RETURN
        END

