!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE cdsj 
!CCC Purpose    : To convolute vib. and rot. density and sum of states 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE cdsj_arot(SS,DS,T,AT,DELE,Maxi,B,J)

        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)

!!!  SS         : sum of states at a pair (E,J)
!!!  DS         : density of states
!!!  T          : number of states within a bin
!!!  AT         : temperary vector
!!!  DELE       : energy bin (grain energy)
!!!  Maxi       : maximum index in master-equation
!!!  B          : three rot. constants
!!!  J          : angular momentum number

        integer Maxi, J
        real(kind=qp) :: SS(Maxi), DS(Maxi)
        integer K,I,N,JJ
        real(kind=qp) :: tp, B(3), Er, DELE, Ev(2000)
        real(kind=qp) :: T(Maxi), AT(Maxi)
        real(kind=qp) :: T2(Maxi), AT2(Maxi)

        IF(J.NE.0) THEN

        N=0
        DO K=-J, J, 1
                tp = Er(B,J,K)
                IF(tp.GE.0.0D0) THEN
                        N = N +1
                        Ev(N) = Er(B,J,K)
                ENDIF
        ENDDO

        DO I=1, Maxi
                T2(I)=T(I)
                AT2(I)=AT(I)
        ENDDO

        CALL EVT(T2,AT2,DELE,Maxi,Ev,N)

        DS(1) = T2(1)/DELE
        SS(1) = T2(1)

        DO I=2, Maxi
                DS(I) = T2(I)/DELE
                SS(I) = SS(I-1) + T2(I)
        ENDDO

        ELSE
        DS(1) = T(1)/DELE
        SS(1) = T(1)

        DO I=2, Maxi
                DS(I) = T(I)/DELE
                SS(I) = SS(I-1) + T(I)
        ENDDO
        ENDIF

        RETURN
        END

