!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE cdsj 
!CCC Purpose    : To convolute vib. and rot. density and sum of states 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE cdsj_hybr(SS,DS,T,AT,DELE,Maxi,B,J)

        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)


!!!  SS         : sum of states at a pair (E,J)
!!!  DS         : density of states
!!!  T          : number of states within a bin
!!!  AT         : temperary vector
!!!  DELE       : energy bin (grain energy)
!!!  Maxi       : maximum index in master-equation
!!!  B          : three rot. constants
!!!  J          : angular momentum number

        integer Maxi, J
        real(kind=qp) :: SS(Maxi), DS(Maxi)
        integer K,I,N,II, JJ
        real(kind=qp) :: tp, B(3), Er1, Er2, DELE, Ev(2000)
        real(kind=qp) :: T(Maxi), AT(Maxi)
        real(kind=qp) :: T2(Maxi), AT2(Maxi)
        real(kind=qp) :: su, tp2, ENG

        IF(J.NE.0) THEN

        N=0
        DO K=-J, J, 1
                tp = Er2(B,J,K)
                IF(tp.GE.0.0D0) THEN
                        N = N +1
                        Ev(N) = Er2(B,J,K)
                ENDIF
        ENDDO

        DO I=1, Maxi
                T2(I)=T(I)
                AT2(I)=AT(I)
        ENDDO

        CALL EVT(T2,AT2,DELE,Maxi,Ev,N)

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        DO I=1, Maxi
                ENG = (I-1)*DELE
                tp2 = Er1(B,J,0)
                tp = ENG - tp2
                DS(I) = 0.0D0
                IF((tp2.GE.0.0D0).and.(tp.GE.0.0D0)) THEN
                        II = INT(tp/DELE) + 1
                        DS(I) = T2(II)/DELE 
                ENDIF
        ENDDO

        SS(1) = DS(1)*DELE
        DO I=2, Maxi
                SS(I) = SS(I-1) + DS(I)*DELE
        ENDDO

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        ELSE
        DS(1) = T(1)/DELE
        SS(1) = T(1)

        DO I=2, Maxi
                DS(I) = T(I)/DELE
                SS(I) = SS(I-1) + T(I)
        ENDDO
        ENDIF

        RETURN
        END

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        real*8 FUNCTION Er1(B,J,K)

        IMPLICIT NONE

        real*8 B(3), tp

        Integer J, K

        tp = sqrt(B(2)*B(3))
        Er1 = tp*J*(J+1) 

        RETURN
        END

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        real*8 FUNCTION Er2(B,J,K)

        IMPLICIT NONE

        real*8 B(3), tp

        Integer J, K

        tp = sqrt(B(2)*B(3))
        Er2 = (B(1)-tp)*K*K

        RETURN
        END

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



