!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE cdsj 
!CCC Purpose    : To convolute vib. and rot. density and sum of states 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE cdsj_inar(SS,DS,T,AT,DELE,Maxi,B,J)

        IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)

!!!  SS         : sum of states at a pair (E,J)
!!!  DS         : density of states
!!!  T          : number of states within a bin
!!!  AT         : temperary vector
!!!  DELE       : energy bin (grain energy)
!!!  Maxi       : maximum index in master-equation
!!!  B          : three rot. constants
!!!  J          : angular momentum number

        integer Maxi, J
        real(kind=qp) :: SS(Maxi), DS(Maxi)
        integer K, I, N, JJ, II
        real(kind=qp) :: su, ENG, tp, tp2, B(3), Er, DELE
        real(kind=qp) :: T(Maxi), AT(Maxi)

        IF(J.NE.0) THEN

        DO I=1, Maxi
                ENG = (I-1)*DELE
                tp2 = Er(B,J,0)
                DS(I) = 0.0D0
                IF(tp2.GE.0.0D0) THEN
                        tp = ENG - tp2
                        IF(tp.GE.0.0D0) THEN
                                II = INT(tp/DELE) + 1
                                DS(I) = T(II)/DELE 
                        ENDIF
                ENDIF

                su = 0.0D0 
                DO K = 1, J
                        tp2 = Er(B,J,K)
                        IF(tp2.GE.0.0D0) THEN
                        tp = ENG - tp2 
                        IF(tp.GE.0.0D0) THEN
                                II = INT(tp/DELE) + 1 
                                su = su + T(II)/DELE 
                        ENDIF
                        ENDIF
                ENDDO
                DS(I) = DS(I) + 2*su 
        ENDDO

        SS(1) = DS(1)*DELE
        DO I=2, Maxi
                SS(I) = SS(I-1) + DS(I)*DELE 
        ENDDO

        ELSE
        DS(1) = T(1)/DELE
        SS(1) = T(1)

        DO I=2, Maxi
                DS(I) = T(I)/DELE
                SS(I) = SS(I-1) + T(I)
        ENDDO
        ENDIF

        RETURN
        END

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


