!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!!CC                                                                                    CC
!!CC                                                                                    CC
!!CC   Parallel Time Dependent Master Equation // Two Dimensional Master Equation       CC
!!CC   "pTDME" is a chemical kinetics program, which can be used to compute thermal     CC
!!CC   rate constants and product yields as functions of both temperature and           CC
!!CC   pressure in parallel using a deterministic (matrix) method.                      CC
!!CC                                                                                    CC
!!CC   Authors: Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)                           CC 
!!CC            John F. Stanton  (jfstanton137@gmail.com)                               CC
!!CC            Department of Chemistry                                                 CC
!!CC            University of Florida in Gainesville, USA.                              CC
!!CC                                                                                    CC
!!CC   Date:    Feb-4-2021                                                              CC
!!CC                                                                                    CC
!!CC   Where:   Gainesville, Florida, USA.                                              CC
!!CC                                                                                    CC
!!CC   Purpose: Education and Research should be encouraged...!!!                       CC
!!CC                                                                                    CC
!!CC                                                                                    CC
!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	Program LAM_pTDME_Feb_4_2021 

        IMPLICIT NONE

        Include 'mpif.h'
        Include 'declare.inc'

integer, parameter :: dp = selected_real_kind(15, 307)

real(kind=dp) ::  Vf(No), Vr(No), Em(No), Ep(No), ETS(No)
real(kind=dp) :: Emax, QT(100), Qts, Qts2, Qre, B2(3), Ev, sigma
real(kind=dp) :: SYMRE(No), SYMTS(No)

!!!  Em, Ep, Ets        : a vector of energies of minima, products, TSs
!!!  Vf, Vr             : a vector of forward and reverse barriers

        Integer MAXJ, NJ

!!!  MAXJ       : maximum angular momentum number
!!!  NJ         : step size of angular momentum

real(kind=dp) :: CON2, CON3 
        Parameter (CON2=4.184D0, CON3=1.0D0)

!!!  CON2       : a convert factor from kcal to kJ

        Integer  Nsink(No), IL(No), KL(No), NsinkTS(No)  

!!!  Nsink      : a vector of sinks to collect stabilized intermediates 
!!!  IL, KL     : a vector of indexes of minima 

        Integer I, J, K, Maxi, KK, LL
        Integer Jmin, Jmax, Np, NJJ
        Integer  NNNN, MMMM, MM

!!!  Maxi       : maximum index used in master-equation 
!!!  NNNN       : angular momentum number (J) to be inputed.
!!!  MMMM       : pressure in reactor to be inputed.

        Integer NE(No+NM), NN, NTS, IND, NETS

!!!  NN         : a number of intermediates
!!!  NTS        : a number of TSs
!!!  IND        : index of the lowest-lying intermediate.

real(kind=dp) ::  Sigf(No), Sigr(No), RKinf

!!!  Sigf, Sigr : vector of forward and reverse reaction path degeneracies.

real(kind=dp), Allocatable :: SS(:,:), TSS(:,:), TSS0(:,:,:)
real(kind=dp), Allocatable :: DS(:,:), TDS(:,:)
real(kind=dp), Allocatable :: T1(:), AT(:), T2(:)
real(kind=dp), Allocatable :: SS1(:), DS1(:)
real(kind=dp), Allocatable :: SS2(:), DS2(:)
real(kind=dp), Allocatable :: FE(:,:), FET(:,:)
real(kind=dp), Allocatable :: SS0(:), DS0(:)

!!!  SS, TSS    : vector of sum and number of states for TSs
!!!  DS, TDS    : vector of sum and number of states for minima
!!!  FE         : initial energy distribution function, F(E,J)

real(kind=dp) :: BT(3), BTS(3), su2, ESP, BAJ

real(kind=dp) ::  CF, PP, betaT, Tt, Tv, rKp, RKc(50), DENS, RKK
real(kind=dp) ::  CF2, E0, EJ, ENG, Edown, DELE, AE, AJ, RKc2(50), RKcc 

!!!  Edown      : down energy transfer (cm-1)
!!!  DELE       : bin energy (grain energy) (cm-1)
!!!  RKc        : overall bi-molecular reaction (cm3/molecule/s) 
!!!  rKp        : collision frequency (1/s) at a particular pressure, PP (atm) 
!!!  betaT       : = 1/RT (cm-1)
!!!  T          : temperature (K)
!!!  PP         : individual pressure (Torr)
!!!  CF         : collision frequency (1/s) at input pressure, Press (atm)


INTEGER, Allocatable :: NN0(:,:) 


real(kind=dp) ::  su, suT, suJ, suJT, suk 

real(kind=dp) ::  Press, m1, Dc1, xma1, m2, Dc2, xma2
real(kind=dp) ::  PMMMM
real(kind=dp) ::  Tstart, Tstop, Tim0

!!!  Press      : pressure (atm); often set Press = 1 atm
!!!  m1, m2     : masses of bath gas and intermediates/nascent adducts (g/mol)
!!!  Dc1, xma1  : Lennard-Jones collision parameters for bath gas: Sigma (Angstrom) and Eps/kB (K)
!!!  Dc2, xma2  : Lennard-Jones collision parameters for intermediates/nascent
!adducts

        CHARACTER*10 CHA 
        CHARACTER*27 CHAC1 
        CHARACTER*17 CHAC2
        CHARACTER*20 CNAME(NM,No)
        CHARACTER*20 CNM(No), CNP(No), CNTS(No)
        CHARACTER*20 CNMabc(No), CNTSabc(No)
        CHARACTER*3 FEINT, FEDISS
        CHARACTER*5 ESOLVER, ENTRANCETS
        CHARACTER*12 CDSM(No), CDSTS(No)

        Integer NDISS, II, JJ, JJJ
        Integer NDI(NM), NDJ(No)

        Integer NDOF, MDOF, MX, MY
real(kind=dp) ::  FREQ(200) 
real(kind=dp) ::  BL, UL, RC(1000), EC(1000), Vff, Vrr
real(kind=dp) ::  Step, Eint, IFREQ, SFR, tp
        CHARACTER*3 CFR
real(kind=dp), Allocatable :: FW(:,:)        
        Character*4 MODELC
        Character*8 MODELJ, MLIN(50), tpC
        Character*6 MODELR
        Character*6 MODELDA

real(kind=dp) :: SE90(9), SE91(9), SE9(9)
real(kind=dp), Allocatable :: STime(:), STime0(:), STime1(:) 
real(kind=dp), Allocatable :: SNN0(:,:), SNN1(:,:), SNN(:,:)
real(kind=dp), Allocatable :: SNP0(:,:,:), SNP1(:,:,:), SNP(:,:,:) 
real(kind=dp), Allocatable :: SRP0(:,:,:), SRP1(:,:,:), SRP(:,:,:) 
         Integer myrank, nprocs, ierr
         Integer ista, iend
         Integer NTC, N1, N2


!!!  CHA                : energy unit of minima, products, and TSs (kJ, or kcal, or cm-1)

!!!  CNM, CNP, CNTS     : names of minima, products, and TSs, which contain
!density and sum of states.
!!!  CNMabc, CNTSabc    : names of files that contain three rot. constants.
!!!  FEINT              : either "YES" or "NO"
!!!  ESOLVER            : either "DENSM" or "SPASM"
!!!  FEDISS             : either "YES" or "NO"
!!!  NDISS              : a number of F(E)J energy distribution fucntions at the
!moment of dissociation

!!!        read(100,*) PMMMM   !!! Considered pressure in Torr
!!!        read(101,*) NNNN   !!! Considered angular momentum number, J

        open (unit=1, file='pTDME.INP',status='old')

!!!  TDME.INP   : main input file

        read(1,*) PMMMM

        Read(1,*) Emax, DELE, MODELR, Ev            !!! Line 1 : INPUT DATA
        Maxi = INT(Emax/DELE) + 1

        sigma = Ev

        Read(1,*) MAXJ, NJ             !!! Line 2 : INPUT DATA

        MODELJ="INACTIVE"

        Read(1,*) Tt, Press, Edown      !!! Line 3 : INPUT DATA

        NJJ = 0

        Read(1,*) m1, Dc1, xma1         !!! Line 4 : INPUT DATA
        Read(1,*) m2, Dc2, xma2         !!! Line 5 : INPUT DATA
        Read(1,*) RKc(1), MODELC                   !!! Line 6 : INPUT DATA

        RKcc = RKc(1)*1.0D+15  !!! [C] = 1.0E+15 1/cm3 is assumed for
!!!                co-reactant, of which concentration remains constant.

        write(*,*) ""
        write(*,*) ""
        write(*,*) "Conditions (T,P,J) in Chemical Reactor : "
        write(*,*) "Temperature (K)         = ", Tt
        write(*,*) "Pressure (Torr)         = ", PMMMM
        write(*,*) "Max. Angular momentum   = ", MAXJ
        write(*,*) "A step size of J        = ", NJ 
        write(*,*) ""
        write(*,*) ""

!!!  To compute a Collision Frequency, CF

        CALL CalFC(CF,Tt,Press,m1,Dc1,xma1,m2,Dc2,xma2,MODELC) 

        CHAC1 = "Collision frequency (1/s)="
        CHAC2 = ", calculated at:"

        write(*,991) CHAC1, CF, CHAC2, Press, "atm & ", Tt, "K" 

991     FORMAT(A,1X,ES12.4,1X,A,1X,F10.2,1X,A,F10.2,1X,A)

        betaT = 1.D0/(RT*Tt)

        PP=PMMMM/760.0D0  !!! convert Torr to atm
        rKp = CF*PP            !!! CF at a considered pressure, PP 

        ESP = rKp/1.0D+13  !!! for Chemically Reaction

        ALLOCATE ( SS0(Maxi), DS0(Maxi) )
        ALLOCATE ( T1(Maxi), AT(Maxi), T2(Maxi) )
        ALLOCATE ( SS1(Maxi), DS1(Maxi) )
        ALLOCATE ( SS2(Maxi), DS2(Maxi) )
        ALLOCATE( FE(Maxi,0:MAXJ), FET(Maxi,0:MAXJ) )


!!!!!!!!!!!!
!!!!!!!!!!!!
!! INPUT for Minima

        Read(1,*) NN, Np, CHA           !!! Line 7 : INPUT DATA

        ALLOCATE ( DS(NN,Maxi), TDS(NN,Maxi) )

        DO I=1, NN
                Read(1,*) J, CDSM(I), CNM(I), CNMabc(I), Em(I), SYMRE(I), tp, MLIN(I)   !!! Line 8 : INPUT DATA
                SYMRE(I) = SYMRE(I)/tp
                IF(CHA.EQ.'kJ') THEN
                        Em(I) = Em(I)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        Em(I) = Em(I)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        Em(I) = Em(I)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF
        ENDDO
        DO I=1, Np
                Read(1,*) J, CNP(I), Ep(I+NN)              !!! Line 9 : INPUT

                IF(CHA.EQ.'kJ') THEN
                        Ep(I+NN) = Ep(I+NN)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        Ep(I+NN) = Ep(I+NN)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        Ep(I+NN) = Ep(I+NN)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF
        ENDDO

        DO I=1, NN
                DO J=1, Np+1
                IF(J.eq.1) THEN
                        CNAME(I,J) = CNM(I)
                ELSE
                        CNAME(I,J) = CNP(J-1)
                ENDIF
                ENDDO
        ENDDO 

        E0 = Em(1)
        IND = 1
        DO I=2, NN
                IF(E0.GT.Em(I)) THEN
                        E0 = Em(I)
                        IND = I
                ENDIF
        ENDDO

!!        write(*,*) ""
!!        write(*,*) "Minimum Index and Its Position in Energy-Grained ME:"
        DO I=1, NN
                ENG = Em(I) - E0
                NE(I) = NINT(ENG/DELE) + 1
                write(*,*) I, NE(I)
        ENDDO
        write(*,*) ""

        DO I=1, NN      !!! BEGIN of DO-LOOP for minima

        IF(CDSM(I).EQ.'EXTERNAL') THEN          !!! EXTERNAL

        open (unit=10+I, file=CNM(I),status='old')
                DO J=1, Maxi
                        Read(10+I,*) K, ENG, DENS, DS(I,J) !!! INPUT : dens/sums 
                ENDDO
        close (unit=10+I, status='keep')

        ELSE IF(CDSM(I).EQ.'CRRKM_HO') THEN     !!! CRRKM_HO
        open (unit=10+I, file=CNM(I),status='old')
                Read(10+I,*) NDOF
                DO J=1, NDOF
                        Read(10+I,*) FREQ(J)
                ENDDO
        close (unit=10+I, status='keep')                         
      
                CALL driver_CRRKM( SS0, DS0, Maxi, DELE, NDOF, FREQ ) 
                DO J=1, Maxi
                        DS(I,J) = SS0(J)
                ENDDO
        ELSE
                write(*,*) "INPUT DATA WRONG for INTERMEDIATE / WELL: ", I
        ENDIF 
 
        open (unit=1000+I, file=CNMabc(I),status='old')
                DO J=1, 3
                        Read(1000+I,*) BB(I,J)  !!! INPUT : three rot. constants
                ENDDO
        close (unit=1000+I, status='keep') 
        ENDDO  !!! END of DO-LOOP for minima

        DO I=1, NN
                DO J=1, Maxi
                        IF(J.LT.NE(I)) THEN
                                TDS(I,J) = 0.0D0
                        ELSE IF(J.EQ.NE(I)) THEN
                             TDS(I,J) = DS(I,J-NE(I)+1)
                        ELSE
                             TDS(I,J) = DS(I,J-NE(I)+1) - DS(I,J-NE(I))
                        ENDIF
                ENDDO

!!!!!!!!!  
!!!   Set SINK for each stable INTERMEDIATE
!!!!!!!!!

        Nsink(I) = NE(I) 

        ENDDO

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
!! J-effect on <DE>d = <DE>d_vib + <DE>d_rot
!! <DE>d_rot = <B> x Del_J x (2*J + 1 - Del_J)
!! Spherical top
!!
        BAJ = 1.0d0
        DO I=1, 3
                BAJ = BAJ * BB(IND,I)
        ENDDO
        BAJ = (BAJ)**(1.0d0/3)

        Edown = Edown + BAJ*NJJ*(2*NNNN+1-NJJ)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!! INPUT for TSs

        Read(1,*) NTS, CHA              !!! Line 10 : INPUT 

        ALLOCATE ( SS(NTS,Maxi), TSS(NTS,Maxi) )
        ALLOCATE ( TSS0(NTS,Maxi,MAXJ/NJ+1) )


        DO I=1, NTS
        Read(1,*) J,CDSTS(I),CNTS(I),CNTSabc(I),IL(I),KL(I),ETS(I),SYMTS(I),tp,MLIN(I+NN) !!! Line 11 : INPUT

                SYMTS(I) = SYMTS(I)/tp

                IF(CHA.EQ.'kJ') THEN
                        ETS(I) = ETS(I)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        ETS(I) = ETS(I)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        ETS(I) = ETS(I)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF

                Vf(I) = ETS(I) - Em(IL(I))
                IF(KL(I).GT.NN) THEN
                        Vr(I) = ETS(I) - Ep(KL(I))
                ELSE
                        Vr(I) = ETS(I) - Em(KL(I))
                ENDIF

!!!        write(*,*) I, Vf(I)*CON2/con, Vr(I)*CON2/con

        ENDDO

!!!!!!!!CCCCCCCCCCCCCCCCCCCCCCCCC

        DO I=1, NTS     !!! BEGIN of DO-LOOP for TSs

        IF(CDSTS(I).EQ.'EXTERNAL') THEN         !!! EXTERNAL

        open (unit=10+I+NN, file=CNTS(I),status='old')
                DO J=1, Maxi   !!! INPUT : dens/sums for TSs
                        Read(10+I+NN,*) K, ENG, DENS, SS(I,J) 
                ENDDO
        close (unit=10+I+NN, status='keep')

        ELSE IF(CDSTS(I).EQ.'CRRKM_HO') THEN            !!! CRRKM_HO
        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) NDOF
                DO J=1, NDOF
                        Read(10+I+NN,*) FREQ(J)
                ENDDO
        close (unit=10+I+NN, status='keep')

                CALL driver_CRRKM( SS0, DS0, Maxi, DELE, NDOF, FREQ )       
                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO
        ELSE IF(CDSTS(I).EQ.'VRRKM_HO') THEN            !!! VRRKM_HO

        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) MX, BL, UL 
                Read(10+I+NN,*) MY, Step, Eint, CHA
                Read(10+I+NN,*) CFR, SFR

                ALLOCATE ( FW(MX,MY+4) )

                DO J=1, MY + 4 
                        Read(10+I+NN,*) (FW(K,J), K=1, MX)
                ENDDO

        IF(CHA.EQ.'kJ') THEN       !!! convert to kcal from kJ
                Eint = Eint/CON2
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/CON2
                ENDDO
        ELSE IF(CHA.EQ.'kcal') THEN   !!! kcal to kcal
                Eint = Eint/CON3
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/CON3
                ENDDO
        ELSE IF(CHA.EQ.'cm-1') THEN  !!! cm-1 to kcal
                Eint = Eint/con
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/con
                ENDDO
        ELSE
                write(*,*) "UNIT of ENERGY INPUT is WRONG"
                STOP
        ENDIF

        close (unit=10+I+NN, status='keep')


        IF(I.EQ.1) THEN
        QT(I+NN)=0.0D0
        DO J=0, MAXJ, NJ

        JJJ = J/NJ + 1

        CALL driver_aVRRKM( SS0, MX, MY, Maxi, DELE, J, &
                BL, UL, Step, Eint, E0/con, FW, CFR, SFR, MODELJ, MLIN(I+NN))

                DO II=1, Maxi
                ENG = (II-1)*DELE
                FE(II,J) = (2*J+1)*SS0(II)*dexp(-betaT*ENG)
                FET(II,J) = FE(II,J) 
                QT(I+NN) = QT(I+NN) + FE(II,J)
                ENDDO

                TSS0(I,1,JJJ) = SS0(1)
                DO II=2, Maxi
                        TSS0(I,II,JJJ) = SS0(II) - SS0(II-1)
                ENDDO
        ENDDO
        QT(I+NN) = QT(I+NN)*DELE*NJ*betaT*exp(-E0*betaT)
        ELSE
        DO J=0, MAXJ, NJ
        JJJ = J/NJ + 1

        CALL driver_aVRRKM( SS0, MX, MY, Maxi, DELE, J, &
                BL, UL, Step, Eint, E0/con, FW, CFR, SFR, MODELJ, MLIN(I+NN))

                TSS0(I,1,JJJ) = SS0(1)
                DO II=2, Maxi
                        TSS0(I,II,JJJ) = SS0(II) - SS0(II-1)
                ENDDO
        ENDDO
        ENDIF

        DEALLOCATE ( FW )


        ELSE IF(CDSTS(I).EQ.'QRRKM_Eckart') THEN        !!! QRRKM_Eckart
 
        open (unit=10+I+NN, file=CNTS(I),status='old')

                Read(10+I+NN,*) NDOF, IFREQ
                DO J=1, NDOF
                        Read(10+I+NN,*) FREQ(J)  
                ENDDO
        close (unit=10+I+NN, status='keep')

        CALL driver_QRRKM_Eckart( SS0, DS0, Vf(I), Vr(I), Maxi, DELE, &
                                NDOF, FREQ, IFREQ )

                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO

222    FORMAT(I5,2X,F10.2,2X,2(ES15.8,2X))

        ELSE IF(CDSTS(I).EQ.'QRRKM_WKB') THEN           !!! QRRKM_WKB

        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) NDOF, MDOF
                DO J=1, NDOF
                        Read(10+I+NN,*) RC(J), EC(J)
                ENDDO
        close (unit=10+I+NN, status='keep')

        Vff = Vf(I) / 349.754D0 / 627.5095D0
        Vrr = Vr(I) / 349.754D0 / 627.5095D0

        CALL driver_QRRKM_WKB( SS0, DS0, Vff, Vrr, Maxi, DELE, NDOF, FREQ, MDOF, &
                RC, EC ) 

                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO

        ELSE
                write(*,*) "INPUT DATA WRONG"
        ENDIF

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        open (unit=1000+I+NN, file=CNTSabc(I),status='old')
                DO J=1, 3     !!! INPUT : three rot. constants for TSs
                        Read(1000+I+NN,*) BBT(I,J)
                ENDDO
        close (unit=1000+I+NN, status='keep')
        ENDDO                           !!! END of SELECTION

        DO I=1, NTS
                IF(SS(I,1).EQ.1.0D0) THEN
                        ENG = ETS(I) - E0
                        NE(I+NN) = NINT( ENG/DELE ) + 1
                ELSE IF(Vf(I).GT.Vr(I)) THEN
                        ENG = ETS(I) - Vr(I) - E0
                        NE(I+NN) = NINT( ENG/DELE ) + 1
                ELSE
                        ENG = ETS(I) - Vf(I) - E0
                        NE(I+NN) = NINT( ENG/DELE) + 1
                ENDIF

!!! New addition ---Apr 16, 2016

                ENG = ETS(I) - E0
                NsinkTS(I) = NINT( ENG/DELE ) + 1 

!!! END

        ENDDO

        DO I=1, NTS
                IF(CDSTS(I).NE.'VRRKM_HO') THEN
                DO J=1, Maxi
                        IF(J.LT.NE(I+NN)) THEN
                                TSS(I,J) = 0.0D0
                        ELSE IF(J.EQ.NE(I+NN)) THEN
                                TSS(I,J) = SS(I,J-NE(I+NN)+1)
                        ELSE
                                TSS(I,J) = SS(I,J-NE(I+NN)+1) - SS(I,J-NE(I+NN))
                        ENDIF
                ENDDO
                ENDIF
        ENDDO

!!!!!!!!!!!!
!!!!!!!!!!!!

        Read(1,*) ESOLVER, MODELDA                      !!! Line 12 : INPUT DATA
        Read(1,*) FEINT, ENTRANCETS                     !!! Line 13 : INPUT DATA

!!!        Read(1,*) FEDISS, NDISS                         !!! Line 14 : INPUT DATA
!!!        IF(FEDISS.EQ.'YES') THEN
!!!                DO I=1, NDISS
!!!                        Read(1,*) J, NDI(I), NDJ(I)     !!! Line 15 : INPUT DATA
!!!                ENDDO
!!!        ENDIF

        Read(1,*) Tstart, Tstop                         !!! Line 16 : INPUT DATA

        close (unit=1, status='keep')

!!CCCCCCCCCCCCCCCCCCCCCCCC

        IF(FEINT.EQ.'NO') THEN  !!! To compute F(E,J)

        IF((MODELR.EQ.'CAT1').OR.(MODELR.EQ.'CAT2')) THEN  !!! For Chemically Activated Reaction

!!
!! TS1 is assumed to be the transition state to be used to build F(E,J)
!! INT1 (or WELL1) is assummed to be the first intermediate (or well), which is
!! produced by the association of two initial reactants 
!!

        IF(ENTRANCETS.EQ.'TIGHT') THEN    !!! For a Tight TS

        DO I=1, Maxi
                T1(I) = TSS(1,I)
                AT(I) = 0.0D0
        ENDDO
        DO I=1, 3
                BT(I) = BBT(1,I)
        ENDDO

        AJ = 0.0D0
        suJ = 0.0D0
        suJT = 0.0D0
        AE = 0.0D0
        su2 = 0.0D0

        DO J=0, MAXJ, NJ

        CALL cdsj(SS1,DS1,T1,AT,DELE,Maxi,BT,J,MODELJ,MLIN(1+NN))

        su=0.0D0
        suT = 0.0D0
        Do I=1,Maxi
             ENG = (I-1)*DELE
             FE(I,J) = (2*J+1)*SS1(I)*exp(-betaT*ENG)
             FET(I,J) = (2*J+1)*SS1(I)*exp(-betaT*ENG)

             su = su + FE(I,J)
             suT = suT + FET(I,J)
             AE = AE + ENG*FE(I,J)
             su2 = su2 + FET(I,J)
        Enddo
        AJ = AJ + su*J
        suJ = suJ + su
        suJT = suJT + suT
        ENDDO

        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                FET(I,J) = FET(I,J)/su2
                ENDDO
        ENDDO

        su =0.0D0
        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                su = su + FET(I,J)
                ENDDO
        ENDDO

        write(*,992) "Total Sum of F(E,J) =", su

        AJ = AJ/suJ
        AE = AE/suJ

         write(*,*) ""
         write(*,*) "Expectation values from F(E,J) : "
         write(*,*) "Average J        = ", AJ
         write(*,*) "Average E (cm-1) = ", AE
         write(*,*) ""

!!CCCCCCCCCCCCCCCCCCCCCCCC

        ELSE IF(ENTRANCETS.EQ.'LOOSE') THEN   !!! For a Loose TS 

        su2 = 0.0D0
        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                su2 = su2 + FET(I,J)
                ENDDO
        ENDDO
        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                IF(su2.GT.0.0d0) THEN
                        FET(I,J) = FET(I,J)/su2
                ELSE
                        FET(I,J) = 0.0d0
                ENDIF
                ENDDO
        ENDDO

!!!=======================

        AJ = 0.0D0
        suJ = 0.0D0
        DO J=0, MAXJ, NJ
                su=0.0D0
                Do I=1,Maxi
                        su = su + FET(I,J)
                Enddo
                AJ = AJ + su*J
                suJ = suJ + su
        ENDDO

        AE = 0.0D0
        DO I=1, Maxi
                ENG = (I-1)*DELE
                su = 0.0D0
                DO J=0, MAXJ, NJ
                        su = su + FET(I,J)
                ENDDO
                AE = AE + ENG*su
        ENDDO

        IF(suJ.GT.0.0d0) THEN
                AE = AE/suJ
                AJ = AJ/suJ
        ELSE
                AE = 0.0d0
                AJ = 0.0d0
        ENDIF

         write(*,*) ""
         write(*,*) "Expectation values from F(E,J) : "
         write(*,*) "Average J        = ", AJ
         write(*,*) "Average E (cm-1) = ", AE
         write(*,*) ""

        ENDIF

!!CCCCCCCCCCCCCCCCCCCCCCCC


        ELSE IF(MODELR.EQ.'THERMO') THEN

!!
!! INT1 (or WELL1) is assumed to be the starting INTERMEDIATE to build F(E,J)
!!

        DO I=1, Maxi
                T1(I) = TDS(1,I)
                AT(I) = 0.0D0
        ENDDO
        DO I=1, 3
                BT(I) = BB(1,I)
        ENDDO

!!CCCCCCCCCCCCCCCCCCCCCCC

        AJ = 0.0D0
        suJ = 0.0D0
        suJT = 0.0D0
        AE = 0.0D0

        Qre = 0.0D0

        su2 = 0.0D0
        DO J=0, MAXJ, NJ

        CALL cdsj(SS1,DS1,T1,AT,DELE,Maxi,BT,J,MODELJ,MLIN(1))

        su=0.0D0
        suT = 0.0D0
        Do I=1,Maxi
             ENG = (I-1)*DELE
             FE(I,J) = (2*J+1)*DS1(I)*exp(-betaT*ENG)
             FET(I,J) = (2*J+1)*DS1(I)*exp(-betaT*ENG)

             su = su + FE(I,J)
             suT = suT + FET(I,J)
             AE = AE + ENG*FE(I,J)

           su2 = su2 + FET(I,J)     

        Qre = Qre + (2*J+1)*DS1(I)*exp(-betaT*ENG)

        Enddo
        AJ = AJ + su*J
        suJ = suJ + su
        suJT = suJT + suT
        ENDDO

        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                FET(I,J) = FET(I,J)/su2
                ENDDO
        ENDDO

        su =0.0D0
        DO J=0, MAXJ, NJ
                DO I=1, Maxi
                su = su + FET(I,J) 
                ENDDO
        ENDDO
       
!!!        write(*,992) "Total Sum of F(E,J) =", su
 
992     FORMAT(A20,2X,ES10.2)

        Qre = Qre*DELE*NJ

!!!        write(*,*) "Qre = ", Qre 

        DO K=1, NTS
        IF(IL(K).EQ.1) THEN
        Qts = 0.0D0
        Qts2 = 0.0D0

        IF(CDSTS(K).NE.'VRRKM_HO') THEN

        DO J=0, MAXJ, NJ
            DO I=1, Maxi
                T2(I) = TSS(K,I)
                AT(I) = 0.0D0
            ENDDO
            DO I=1, 3
                BTS(I) = BBT(K,I)
            ENDDO
            CALL cdsj(SS2,DS2,T2,AT,DELE,Maxi,BTS,J,MODELJ,MLIN(K+NN))
            DO I=1, Maxi
               ENG = (I-1)*DELE
               Qts = Qts + (2*J+1)*DS2(I)*exp(-betaT*ENG)
               Qts2 = Qts2 + (2*J+1)*SS2(I)*exp(-betaT*ENG)
            ENDDO 
        ENDDO

!!!                write(*,*) Qts/betaT, Qts2

                Qts = Qts*DELE*NJ
                Qts2 = Qts2*DELE*NJ
                RKc(K) = SYMRE(1)*Qts/Qre/hh/betaT/SYMTS(K)
                RKc2(K) = SYMRE(1)*Qts2/Qre/hh/SYMTS(K)
                write(*,9991) "k(T,P=INF) = ", K, RKc(K), RKc2(K)
        ELSE
                Qts = QT(K+NN)
                RKc2(K) = SYMRE(1)*Qts/Qre/hh/betaT/SYMTS(K)
                write(*,9991) "k(T,P=INF) = ", K, RKc2(K) 
        ENDIF
        ENDIF
        ENDDO

        RKK = RKc2(1)
        DO K=2, NTS
                IF(RKK.GT.RKc2(K)) RKK = RKc2(K)
        ENDDO

                IF(PP.GT.1.0D+4) THEN
                        ESP = RKK / 1.0d+04
                ELSE IF(PP.LT.1.0D-4) THEN
                        ESP = RKK / 1.0d+04
                ELSE
                        ESP = RKK / 1.0D+06
                ENDIF

        write(*,*)
        write(*,*)
        write(*,*) "Reaction pressure and ESP value = " 
        write(*,'(ES15.5,2X,ES15.5)') PP, ESP
        write(*,*)
        write(*,*)

9991    FORMAT(A15,2X,I5,2X,2(ES15.5,2X))

        AJ = AJ/suJ
        AE = AE/suJ

         write(*,*) ""
         write(*,*) "Expectation values from F(E,J) : " 
         write(*,*) "Average J        = ", AJ
         write(*,*) "Average E (cm-1) = ", AE
         write(*,*) ""

        ENDIF


!!!=============================

        ELSE IF(FEINT.EQ.'YES') THEN  !!! F(E)J from an EXTERNAL FILE

                suJT = 0.0D0
                open (unit=2, file='FE.EXTERNAL',status='old')
                        DO I=1, Maxi
                                Read(2,*) J, ENG, FET(I,NNNN)
                                suJT = suJT + FET(I,NNNN)
                        ENDDO
                close (unit=2, status='keep')
        ELSE
                write(*,*) "SOMETHING WRONG with INPUT DATA"
                STOP

        ENDIF

!!!        write(333,'(I5,1X,I5,1X,I5)') INT(MAXJ/NJ)+1, NN, Np

        Jmin = NNNN
        Jmax = NNNN

!!! To compute thermal rate constants (or yields) for each product:

        write(*,*)
        write(*,*) " Collision frequency considered (1/s) = "
        write(*,'(ES15.5)') rKp
        write(*,*)

        MM = NN*(Maxi+Np)
        CF2 = rKp

        IF(RKinf.GT.sigma) RKinf = sigma

        RKinf = sigma

        NTC = INT(LOG10(Tstop)-LOG10(Tstart))
        NTC = NTC*9

!!!====================================

        ALLOCATE ( NN0(NN,Np) )


        II = 0
        DO I=1, NN
                DO J=1, Np
                        II = II + 1
                        NN0(I,J) = II + NN
                ENDDO
        ENDDO

!!!====================================

        ALLOCATE ( SNN0(NN,NTC), SNN1(NN,NTC), SNN(NN,NTC) )
        ALLOCATE ( SNP0(NN,NN+Np,NTC), SNP1(NN,NN+Np,NTC), SNP(NN,NN+Np,NTC))
        ALLOCATE ( SRP0(NN,NN+Np,NTC), SRP1(NN,NN+Np,NTC), SRP(NN,NN+Np,NTC) )
        ALLOCATE ( STime1(NTC), STime0(NTC), STime(NTC) )

!!!====================================

        DO II=1, 9
                SE91(II) = 0.0d0
        ENDDO

        IF(MODELDA.EQ."LAPACK") THEN
        DO II=1, NTC
                STime1(II) = 0.0d0
        DO J=1, NN
                SNN1(J,II) = 0.0d0
        DO K=1, NN+Np
                SNP1(J,K,II) = 0.0d0
                SRP1(J,K,II) = 0.0d0
        ENDDO
        ENDDO
        ENDDO
        ENDIF

         CALL MPI_INIT(ierr)
         CALL MPI_COMM_SIZE(MPI_COMM_WORLD,nprocs,ierr)
         CALL MPI_COMM_RANK(MPI_COMM_WORLD,myrank,ierr)
         CALL para_range(1,MAXJ/NJ+1,nprocs,myrank,ista,iend)

         DO I=ista, iend

         Jmin = (I-1)*NJ
         Jmax = (I-1)*NJ
 
         JJJ = Jmin/NJ + 1
 
         DO II=1, NTS
                IF(CDSTS(II).EQ.'VRRKM_HO') THEN
                DO JJ=1, Maxi
                         TSS(II,JJ) = TSS0(II,JJ,JJJ)
                ENDDO
                ENDIF
         ENDDO

        Tim0 = Tstart
        DO II=1, 9
                SE90(II) = 0.0d0
        ENDDO
        
        IF(MODELDA.EQ."LAPACK") THEN
        DO II=1, NTC
                STime0(II) = 0.0d0
        DO J=1, NN
                SNN0(J,II) = 0.0d0
        DO K=1, NN+Np
                SNP0(J,K,II) = 0.0d0
                SRP0(J,K,II) = 0.0d0
        ENDDO
        ENDDO
        ENDDO
        ENDIF

!!!====================================

        IF(MODELR.EQ.'CAT2') THEN
        CALL CALRATE_ASYM(Jmin,Jmax,MAXJ,NJ,DELE,Maxi,FET,NN,NTS,Np,IND,Edown,Tt,Nsink,suJT,CF2,RKc,&
        ESOLVER, BB, BBT, TSS, TDS, CNAME, CNM, CNP, IL, KL, SYMRE, SYMTS,&
                FEDISS, NDISS, NDI, NDJ, CDSTS, Rate_inf, NsinkTS, ESP,&
                Tim0, Tstop, MM, MODELJ, MODELR, RKcc, Ev, MLIN, RKinf, MODELDA, &
                SE90, NTC, SNN0, SNP0, SRP0, STime0)
        ELSE IF(ESOLVER.EQ.'SYMM') THEN
        CALL CALRATE_SYMM(Jmin,Jmax,MAXJ,NJ,DELE,Maxi,FET,NN,NTS,Np,IND,Edown,Tt,Nsink,suJT,CF2,RKc,&
        ESOLVER, BB, BBT, TSS, TDS, CNAME, CNM, CNP, IL, KL, SYMRE, SYMTS,&
                FEDISS, NDISS, NDI, NDJ, CDSTS, Rate_inf, NsinkTS, ESP,&
                Tim0, Tstop, MM, MODELJ, MODELR, RKcc, Ev, MLIN )
        ELSE IF(ESOLVER.EQ.'ASYM') THEN
        CALL CALRATE_ASYM(Jmin,Jmax,MAXJ,NJ,DELE,Maxi,FET,NN,NTS,Np,IND,Edown,Tt,Nsink,suJT,CF2,RKc,&
        ESOLVER, BB, BBT, TSS, TDS, CNAME, CNM, CNP, IL, KL, SYMRE, SYMTS,&
                FEDISS, NDISS, NDI, NDJ, CDSTS, Rate_inf, NsinkTS, ESP,&
                Tim0, Tstop, MM, MODELJ, MODELR, RKcc, Ev, MLIN, RKinf, MODELDA, &
                SE90, NTC, SNN0, SNP0, SRP0, STime0)
        ELSE
                write(*,*) "WRONG INPUT at SELECTION of MATRIX"
        ENDIF

        DO J=1, 9
                SE91(J) = SE91(J) + SE90(J)
!!!                SE90(J) = 0.0d0
        ENDDO

        IF(MODELDA.EQ."LAPACK") THEN

         DO II=1, NTC
                STime1(II) = STime1(II) + STime0(II)
!!!                STime0(II) = 0.0d0
         DO J=1, NN
                 SNN1(J,II) = SNN1(J,II) + SNN0(J,II)
!!!                SNN0(J,II) = 0.0d0 
         DO K=1, NN+Np
                 SNP1(J,K,II) = SNP1(J,K,II) + SNP0(J,K,II)
                 SRP1(J,K,II) = SRP1(J,K,II) + SRP0(J,K,II)
!!!                SNP0(J,K,II) = 0.0d0
!!!                SRP0(J,K,II) = 0.0d0
         ENDDO    
         ENDDO
         ENDDO
        ENDIF


        ENDDO   !!! I-Loop

!!!====================================


        CALL MPI_REDUCE(SE91,SE9,9,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)

        IF(myrank.eq.0) THEN
        write(999,*) ""
        write(999,*) "9 Lowest-Lying Eigenvalues: "
        DO I=1, 9
                write(999,1001) I, SE9(I)
        ENDDO 
        ENDIF
        write(999,*) ""

1001    FORMAT (I5,2X,ES15.8)

        IF(MODELDA.EQ."LAPACK") THEN

        CALL MPI_REDUCE(STime1,STime,NTC,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)

        N1 = NN*NTC
        N2 = NN*NTC*(NN+Np)

        CALL MPI_REDUCE(SNN1,SNN,N1,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)
        CALL MPI_REDUCE(SNP1,SNP,N2,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)
        CALL MPI_REDUCE(SRP1,SRP,N2,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,ierr)


        IF(myrank.eq.0) THEN
        DO I=1, NTC
        II = 0
        JJ = 0
        DO J=1, NN
                JJ = JJ + 1
                write(8000+JJ,1002) STime(I), J, J, SNN(J,I)
        DO K=1, NN+Np
                tp = SNP(J,K,NTC/2)
                IF((K.GT.NN).AND.(tp.GT.0.0d0)) THEN
                JJ = JJ + 1
                write(8000+JJ,1002) STime(I), J, K, SNP(J,K,I)
                ENDIF
                tp = SRP(J,K,NTC/2)
                IF((J.NE.K).AND.(tp.GT.0.0d0)) THEN
                II = II + 1
                write(9000+II,1002) STime(I), J, K, SRP(J,K,I)
                ENDIF
        ENDDO
        ENDDO
        ENDDO
        ENDIF
        ENDIF

1002    FORMAT (ES10.2,2X,2(I5,2X),ES15.8)

        CALL MPI_FINALIZE(ierr)

!!!====================================

        DEALLOCATE ( T1, AT, T2, SS1, DS1, SS2, DS2 )
        DEALLOCATE ( SS, DS, TSS, TDS, SS0, DS0 )
        DEALLOCATE ( FE, FET )
        DEALLOCATE ( TSS0, NN0 )
        DEALLOCATE ( SNN, SNN0, SNN1 )
        DEALLOCATE ( SNP, SNP0, SNP1 )
        DEALLOCATE ( SRP, SRP0, SRP1 )
        DEALLOCATE ( STime1, STime0, STime )

	END Program LAM_pTDME_Feb_4_2021 



